#ifndef __actionhandler_h__
#define __actionhandler_h__

namespace hm {

	// forward declaration
	class Menu;

	/**

		The ActionHandler class is a skeleton class for handling events from
		the menu system. Simply override the methods in a child class and hook 
		your eventhandler to a Menu item.

		The change event is fired when a value is changed, for instance when a 
		checkbox is flipped or a selectbox closes
		
		The action event is fired when a file is selected in the filebrowser or 
		'accept' is pressed over an ActionMenu

	*/
	class ActionHandler {
	public:

		/** Action Event 
			\param source the Menu object that fired the event
		*/
		virtual int action(Menu* source);

		/** Value change event
			\param source the Menu object that fired the event
		*/
		virtual void change(Menu* source);

		/** Enter event, fired when the menu is entered.
			\param source the Menu object that fired the event
		*/
		virtual void enter(Menu* source);
	};

};

#endif
