#ifndef __actionmenu_h__
#define __actionmenu_h__

#include "menu.h"
#include "actionhandler.h"

namespace hm {

	/**

		This is what you use when you wish to add an action to your menu system, 
		like 'reset' or 'save state' for instance. You can hook an eventhandler
		to the Menu object to catch action events. You can also make the menu
		system ask the user for confirmation before firing the action event
		by setting the confirm flag.

	*/
	class ActionMenu : public Menu {

	public:
		
		/** 
			constructor
			\param title the title of the menu, it will usually show in the list and on the window header
			\param surface a surface to draw on. The root node should always have a non-NULL Surface object
			specified, child nodes will inherit their parent's Surface when appendChild is called.
		*/
		ActionMenu(string title,Surface *surface = NULL);

		virtual int enter();

		/** Indicates wether the system should ask for confirmation before firing the action event.
			\param conf if true, a popup will show asking the user to confirm
			\param text the text to appear in said popup
		*/
		void setConfirm(bool conf,string text = "Sure?");

	protected:

		void draw();
		bool confirm;
		string confirmText;
		int selectedButton;
		bool down;

	};


};

#endif
