#ifndef __filemenu_h__
#define __filemenu_h__

#include "menu.h"
#include "filemenuitem.h"

using namespace std;

namespace hm {

	/**
		A simple filebrowser implementation. 
	*/
	class FileMenu : public Menu {

	public:

		/** 
			constructor
			\param title the title of the menu, it will usually show in the list and on the window header
			\param surface a surface to draw on. The root node should always have a non-NULL Surface object
			specified, child nodes will inherit their parent's Surface when appendChild is called.
		*/
		FileMenu(string title,Surface *surface = NULL);

		/**
			Destructor
		*/
		virtual ~FileMenu();

		/**
			Adds a known extension to the filebrowser. For instance, if you want to graphically represent a .gb file by
			showing a cute gameboy icon in front of the filename.
			\param extension string. For instance ".zip"
			\param icon an Image object to represent the file type. Will be shown in front of the filename.
		*/
		void addFileExtension(string extension,Image *icon);

		/** Indicates wether the system should ask for confirmation before firing the action event.
			\param confirm if true, a popup will show asking the user to confirm
			\param text the text to appear in said popup
		*/
		void setConfirm(bool confirm,string text = "");

		/**
			\param ignore if this is set to true, unknown filetypes will not be shown
		*/
		void setIgnoreUnknownTypes(bool ignore);

		virtual int enter();
		
	private:

		void draw();

		/**
			File Extension struct (ext,icon)
		*/
		typedef struct {
			string extension;
			Image *icon;
		} FileExtension;

		vector<FileExtension> fileExensions;

		void openDirectory(string dir);
		void setSelected(int sel);

		Image *directoryIcon,*fileIcon;
		string confirmText;
		bool confirm,ignoreUnknown;

	};


};

#endif

