#ifndef __font_h__
#define __font_h__

#include <string>
#include "image.h"

using namespace std;

namespace hm {

	/**
		The font class is used by the Surface class to draw text. 
	*/
	class Font {

	public:

		/**
			Constructor
			\param fontImage an Image object containing the font information
		*/
		Font(Image* fontImage);

		/**
			Destructor
		*/
		~Font();

		/**
			\param c an ansi character
			\returns <Image> an Image object containg the requested character
		*/
		Image *getLetter(char c);

		/**
			Sets the letter spacing for this font. 
			\param space letterspacing
		*/
		void setLetterSpacing(int space);

		/**
			Sets the word spacing for this font. 
			\param space wordpacing
		*/
		void setWordSpacing(int space);

		/**
			\returns <int> the letters pacing for this font
		*/
		int getLetterSpacing();

		/**
			\returns <int> the word spacing for this font
		*/
		int getWordSpacing();

		/**
			\returns <int> the height of the font
		*/
		int getHeight();

		/**
			Calculates the width of a string in pixels, taking into account word and letterspacing.
			\returns <int> stringwidth
		*/
		int getStringWidth(string text);

	private:

		Image *fontImage;
		Image **letters;
		int height,nrLetters;
		int letterSpacing,wordSpacing;

	};


};

#endif
