#include <string>
#include "surface.h"
#include "radmenu.h"
#include "system.h"
#include "math.h"
using namespace std;

namespace hm
{

	RadMenu::RadMenu(string title,Surface *surface) :
		Menu(title,surface)
	{
		setValue("");
		setWindowSize(120,120);
		selected=0;
		radius = 80;
	}

	void RadMenu::draw()
	{

		Surface *surface = getSurface();

		surface->drawImage(background,0,0);

		this->x = (int) (surface->getWidth()  / 2.0f);
		this->y = (int) (surface->getHeight() / 2.0f);

		double a = -3.14159/2.0f;
		double astep = (3.141592 * 2.0f) / (double)children.size();

		for (int i=0; i<children.size(); i++)
		{

			int x = (int)((cos(a)*radius) + this->x);
			int y = (int)((sin(a)*radius) + this->y);

			a += astep;

			Image *icon=NULL;
			if (selected==i) icon = children.at(i)->getSecondaryIcon();
			if (icon==NULL) icon = children.at(i)->getIcon();

			if (icon==NULL)
			{
				surface->drawText("no icon",x,y);
			}
			else
			{
				surface->drawImage(icon,x-icon->getWidth()/2,y-icon->getHeight()/2);
			}

		}

		surface->flip();

	}

	int RadMenu::enter()
	{
		int res = 0;

		// fire enter event
		if (actionHandler!=NULL)
			actionHandler->enter(this);

		background = getSurface()->toImage();

		draw();
		scroll = 0;

		running=true;
		while (running)
		{
			Event e = msystem.pollEvent();
			if (e.type==EVENT_QUIT) running = false;

			if (e.type==EVENT_KEYUP)
			{
				if (e.key==KEY_REJECT)
					running=false;

				if (e.key==KEY_ACCEPT)
				{
					if (children.size()>selected)
					{
						res = children.at(selected)->enter();
						if (res)
							break;
					}
					draw();
				}
			}

			if (e.type==EVENT_KEYDOWN)
			{
				if (e.key==KEY_DOWN)
					selected=(selected+1)%children.size();

				if (e.key==KEY_UP)
					selected=(selected+children.size()-1)%children.size();

				draw();
			}

		}

		delete(background);
		background = NULL;

		return res;
	}

	void RadMenu::setRadius(int rad)
	{
		radius = rad;
	}

};
