#include <string>
#include "surface.h"
#include "selectmenu.h"
#include "system.h"
using namespace std;

namespace hm
{


	SelectMenu::SelectMenu(string title,Surface *surface) :
		Menu(title,surface)
	{
		setValue("");
		setWindowSize(120,120);
	}

	void SelectMenu::drawListItem(Surface *surface,int x,int y,int w,int h,bool selected)
   {
		
		x+=listItemPadding;
		y+=listItemPadding;

		if (icon!=NULL)
      {
			surface->drawImage(icon,x,y);
			x += icon->getWidth() + listItemPadding;
		}

		surface->drawText(title + ": " + getValue(),x,y,selected?SELECTED:NORMAL);
	}

	int SelectMenu::enter()
   {
      int res = 0;

		// fire enter event
		if (actionHandler!=NULL)
			actionHandler->enter(this);

		// init
		draw();
		scroll = 0;

		background = getSurface()->toImage();

		running=true;
		while (running)
		{
			Event e = msystem.pollEvent();
			if (e.type==EVENT_QUIT) running = false;

			if (e.type==EVENT_KEYUP)
         {
				if (e.key==KEY_REJECT)
					running=false;

				if (e.key==KEY_ACCEPT)
            {

					value = children.at(selected)->getTitle();

					if (actionHandler!=NULL)
						actionHandler->change(this);
					running=false;
				}
			}

			if (e.type==EVENT_KEYDOWN)
         {
				if ((e.key==KEY_DOWN)&&(selected<children.size()-1))
					selected++;

				if ((e.key==KEY_UP)&&(selected>0))
					selected--;

				draw();
			}

		}

		delete(background);
		background = NULL;

      return res;
	}

	void SelectMenu::addOption(string option)
   {
		if (getValue() == "") setValue(option);
		Menu *temp = new Menu(option);
		appendChild(temp);
	}

};
